"""py2ch Dat.
"""
#
# Copyright (c) 2005 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#

import os
import re
import urllib

import config
import dolib
from monazilla import *

__all__ = ["Dat", "DatList"]

class Dat(URL):
    """2ch Thread."""
    def __init__(self, urlstr, title=""):
        URL.__init__(self, urlstr)
        self.title = title

    def _get(self, new=False, diff=False):
        """Get file from network."""
        if not (diff and self.size):
            return URL._get(self, new=False, diff=False)
        else:
            urlopener = Monazilla()
            urlopener.addheader("Range", "bytes=%d-" % (self.size-1))
            f = urlopener.open(self.urlstr)
            buf = f.read()
            if f.info().getheader("Content-Encoding", "") == "gzip":
                buf = gunzip(buf)
            f.close()
            return buf

    def get(self, new=False, diff=False):
        """Get file from network.

        return live-thread-or-not.
        """
        mode = "wb"
        try:
            if diff and self.size:
                mode = "ab"
                try:
                    buf = self._get(diff=True)
                    if diff and self.size and (not buf.startswith("\n")):
                        buf = self._get()
                        mode = "wb"
                    else:
                        buf = buf[1:]
                except HTTPError416:
                    buf = self._get()
                    mode = "wb"
            else:
                buf = self._get(new=new)
            if buf:
                self.write(mode, buf)
            return True
        except (HTTPError302, HTTPError404):
            #if dolib.online():
            #    self.dolibget()
            return False
        except socket.error:
            return False

    def dolibget(self):
        """Get dat using DOLIB.

        return download-succeed or not.
        """
        try:
            mode = "wb"
            found = re.search(r"http://([^/]+)/([^/]+)/dat/(\d+).dat",
                              self.urlstr)
            g = found.groups()
            opt = urllib.urlencode((("raw", "0.0"),
                                    ("sid", dolib.sid)))
            buf = geturl("http://%s/test/offlaw.cgi/%s/%s/?%s" %
                         (g[0], g[1], g[2], opt))
            if buf[0].startswith("+OK"):
                self.write(mode, "\n".join(buf[1:]))
                return True
            else:
                return False

        except (HTTPError302, HTTPError404), err:
            return False
        except socket.error:
            return False


class DatList(dict):
    def __init__(self):
        dict.__init__(self)
        try:
            f = file("%s/title.txt" % config.cachedir)
            for line in f:
                try:
                    urlstr, title = line.strip().split("<>")
                    self[urlstr] = Dat(urlstr, title)
                except ValueError:
                    pass
        except IOError:
            pass

    def sync(self):
        f = file("%s/title.txt" % config.cachedir, "wb")
        for key in self:
            f.write("%s<>%s\n" % (key, self[key].title))
        f.close()

    def remove(self, urlstr):
        """Remove dat from list and disk."""
        try:
            os.remove(self[urlstr].filename)
        except OSError:
            pass
        try:
            del self[urlstr]
        except KeyError:
            pass
        self.sync()


class Cookie(dict):
    def __init__(self):
        dict.__init__(self)
        try:
            f = file("%s/cookie.txt" % config.cachedir)
            for line in f:
                try:
                    key, val = line.strip().split("<>")
                    self[key] = val
                except ValueError:
                    pass
        except IOError:
            pass

    def sync(self):
        f = file("%s/cookie.txt" % config.cachedir, "wb")
        for key in self:
            f.write("%s<>%s\n" % (key, self[key]))
        f.close()
