"""py2ch User Interface.
"""
#
# Copyright (c) 2005 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#

import re
import sys

import config
from dat import *
from monazilla import geturl

script = ""

def html_header(title="", charset="Shift_JIS"):
    """Output HTTP and HTML header."""
    sys.stdout.write(
         "Content-Type: text/html; charset=%s\n\n" % charset +
         '<?xml version="1.0" encoding="%s"?>\n' % charset +
         '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"' +
         '  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">' +
         '<html xmlns="http://www.w3.org/1999/xhtml"' +
         ' lang="ja" xml:lang="ja">' +
         '<head>\n' +
         '  <meta http-equiv="content-type"' +
         ' content="text/html; charset=%s" />\n' % charset+
         '  <title>%s</title>\n' % title +
         '  <link rev="made" href="http://fuktommy.s64.xrea.com/py2ch/" />\n'+
         '  <link rel="contents" href="%s?mode=menu" />\n' % script +
         '</head>\n')

def print403():
    """Output NOT_ALLOWED message."""
    html_header("not allowed")
    sys.stdout.write(
        '<body><p>You are not allowed to use' +
        ' <a href="http://fuktommy.s64.xrea.com/py2ch/">py2ch</a>' +
        '</p>\n</body></html>\n')

def print_menu(url):
    """Output menu - list of boards."""
    html_header("2ch menu")
    sys.stdout.write(
        '<body><ul>\n' +
        '<li><a href="%s?mode=cache">Cache</a></li>\n' % script +
        '<li><a href="%s?mode=tcache">Cache(sorted)</a></li>\n' % script +
        '<li><a href="%s?mode=search">Search</a></li>\n' % script +
        '<li><a href="%s?mode=login">Login</a></li>\n' % script +
        '<li><a href="%s?mode=logout">Logout</a></li>\n' % script)
    flag_comment = False
    start_comment = re.compile(r"<!--").search
    end_comment = re.compile(r"/-->/").search
    istitle = re.compile(r"<B>(.*)</B>", re.I).search
    is2chboard = re.compile(r"<A HREF=http://[^.]*\.2ch\.net/", re.I)
    ispinkboard = re.compile(r"<A HREF=http://[^.]*\.bbspink\.com/", re.I)
    islink = re.compile(r"^(<BR>)*<A HREF.*</A>", re.I).search
    deltarget = re.compile(r' *TARGET="?[^ >"]*').sub
    deltitle = re.compile(r'<B>.*</B>', re.I).sub
    buf = url.open().readlines()
    while buf:
        line = buf.pop(0)
        if start_comment(line) and (not end_comment(line)):
            flag_comment = True
        elif flag_comment and end_comment(line):
            flag_comment = False
        elif flag_comment:
            pass
        elif is2chboard.search(line) or ispinkboard.search(line):
            line = line.strip()
            line = deltitle("", line)
            line = deltarget("", line)
            line = line.replace("<br>", "")
            line = line.replace("<BR>", "")
            line = line.replace('A HREF=', 'a href="%s?'%script)
            if line.find('">') < 0:
                line = line.replace('>', '">', 1)
            line = line.replace("</A>", "</a>")
            line = line.replace("&", "&amp;")
            print '<li>%s</li>' % line
        elif line.find("img src") >= 0:
            pass
        elif islink(line):
            line = line.strip()
            line = deltarget("", line)
            line = line.replace("<br>", "")
            line = line.replace("<BR>", "")
            line = line.replace('A HREF=', 'a href="')
            line = line.replace('>', '">', 1)
            line = line.replace("</A>", "</a>")
            print '<li>%s</li>' % line

        else:
            found = istitle(line)
            if found:
                print "</ul><h3>%s</h3><ul>" % found.group(1)
                buf.insert(0, line[found.end()])

    print "</ul></body></html>";

def print_board(url, hostname, boardname):
    datlist = DatList()
    cached = []
    uncached = []
    for line in url.open():
        dat, title = line.strip().split("<>", 1)
        dat = "http://%s/%s/dat/%s" % (hostname, boardname, dat)
        if dat in datlist:
            cached.append((dat, title))
        else:
            uncached.append((dat, title))
    html_header(boardname)
    print "<body></h1>%s</h1><ul>" % boardname
    for i in cached:
        print '<li><a href="%s?%s">%s</a> cached</li>' % (script, i[0], i[1])
    for i in uncached:
        print '<li><a href="%s?%s">%s</a></li>' % (script, i[0], i[1])
    print "</ul></body></html>";

def escape(s):
    s = s.replace("&", "&amp;")
    s = s.replace("&ampgt;", "&gt;")
    s = s.replace("&amplt;", "&lt;")
    return s

def format(s):
    s = s.replace("<br>", "<br />")
    s = re.sub(r"<a.*?>|<\/a>", "", s)
    s = re.sub(r"(&gt;&gt;|>>|\201\204|\201\164|\201\342|&gt;)(\d+)",
               r'<a href="#r\2">\g<0></a>', s)
    s = re.sub(r"https?://[\041-\073\075-\177]{2,}",
               r'<a href="\g<0>">\g<0></a>', s)
    s = re.sub(r"([^h])(ttps?://[\041-\073\075-\177]{2,})",
               r'\1<a href="h\2">\2</a>', s)
    s = re.sub(r'<a href="(http://[^>]*?\.2ch\.net/[^>]*?)"[^>]*?>',
               r'<a href="%s?\1">' % script, s)
    s = re.sub(r'<a href="(http://[^>]*?\.bbspink\.com/[^>]*?)"[^>]*?>',
               r'<a href="%s\1">' % script, s)
    return s

def print_thread(url, live=True):
    """Print thread.

    live: We can post the tread or not.
    """
    import time
    urlstr = str(url)
    found = re.search(r"http://([^/]+)/([^/]+)/dat/(\d+).dat", urlstr)
    if found:
        host, bbs, key = found.groups()
        link = "http://%s/test/read.cgi/%s/%s/l50" % (host, bbs, key)
    else:
        link = ""
    buf = url.open().readlines()
    if buf:
        tmp = buf[0].split("<>")
        title = escape(tmp[4]).strip()
        datlist = DatList()
        if urlstr not in datlist:
            datlist[urlstr] = Dat(urlstr, title)
            datlist.sync()
        html_header(title)
    else:
        html_header("not found")
        title = "<a href='%s?url=%s'>%s</a>" % (script, link, link)

    print "<body><h1>%s</h1><dl>" % title
    i = 0
    delbe = re.compile(r"<a[^<>]*>[^<>]*</a>").sub
    for line in buf:
        i += 1
        tmp = line.strip().split("<>")
        name, mail, date, message = tmp[:4]
        name = name.replace("<b>", "")
        name = name.replace("</b>", "")
        date = delbe("", date)
        message = format(message)
        print '<dt id="r%d">%d :%s [%s] %s</dt><dd>%s<br /><br /></dd>' % \
              (i, i, name, mail, date, message)
    if i == 0:
        print '<dt>DAT? HTML?</dt><dd><a href="%s">%s</a>' % (link, link)
    print "</dl>"
    if (0 < i) and (i < 1000) and live:
        found = re.search(r"http://([^/]+)/([^/]+)/dat/(\d+).dat", urlstr)
        host, bbs, key = found.groups()
        date = int(time.time())
        sys.stdout.write(
            '<form method="post" action="%s"><p>\n' % script +
            '<input type="submit" value="Post" name="submit"' +
            ' tabindex="1" accesskey="p" />' +
            'Name: <input name="FROM" size="19" value=""' +
            ' tabindex="2" accesskey="n" />' +
            'E-Mail: <input name="mail" size="19" value="sage"' +
            ' tabindex="3" accesskey="m" /><br />\n' +
            '<textarea rows="5" cols="70" name="MESSAGE"' +
            ' tabindex="4" accesskey="c"></textarea>' +
            '<input type="hidden" name="host" value="%s" />' % host +
            '<input type="hidden" name="bbs" value="%s" />' % bbs +
            '<input type="hidden" name="key" value="%s" />' % key +
            '<input type="hidden" name="time" value="%d" />' % date +
            '<input type="hidden" name="mode" value="post" />' +
            '<input type="hidden" name="url" value="%s" />' % urlstr +
            "</p></form>\n")

    sys.stdout.write(
        '<p><a href="%s">read.cgi</a> | ' % link +
        '<a href="%s?mode=update&amp;url=%s">Force-Update</a> | ' %
        (script, urlstr))
    if not ((0 < i) and (i < 1000) and live):
        sys.stdout.write(
            '<a href="%s?mode=viewer&amp;url=%s">Viewer</a> | ' % 
            (script, urlstr))
    sys.stdout.write(
        '<a href="%s?mode=login">Login</a> | ' % script +
        '<a href="%s?mode=remove&amp;url=%s">Remove</a></p>' %
        (script, urlstr) +
        '</body></html>')

def print_login(url):
    urlstr = str(url)
    found = re.search(r"http://([^/]+)/([^/]+)/dat/(\d+).dat", urlstr)
    if found:
        host, bbs, key = found.groups()
        link = "http://%s/test/read.cgi/%s/%s/l50" % (host, bbs, key)
    else:
        link = ""
    html_header("Login")
    sys.stdout.write(
        '<body><p>' +
        '<a href="%s?mode=login">Login</a> | ' % script +
        '<a href="%s?mode=viewer&amp;url=%s">Viewer</a>'% (script, urlstr) +
        '</body></html>')

def print_cache(sort="file"):
    """Print Cache Titles.

    sort=file : sort by filename.
    sort=time : sort by modified time.
    """
    datlist = DatList()
    keys = datlist.keys()
    if sort == "file":
        keys.sort(lambda a,b: cmp(str(datlist[a]), str(datlist[b])))
    elif sort == "time":
        keys.sort(lambda a,b: cmp(datlist[b].mtime, datlist[a].mtime))
    html_header("2ch cache")
    print "<body><h1>2ch cache</h1><ul>"
    for k in keys:
        print '<li><a href="%s?%s">%s</a></li>' % \
              (script, k, datlist[k].title)
    print "</ul></body></html>"

def print_search(key, offset):
    from urllib import quote
    key = key.replace('"', "&quote;")
    if not offset:
        offset = 0
    else:
        offset = int(offset)
    str_key = quote(key)
    html_header("Search", charset="EUC-JP")
    sys.stdout.write(
        '<body><form method="get" action="%s"><p>\n' % script +
        '<input name="key" value="%s" tabindex="1" accesskey="k" />' % key +
        '<input type="submit" name="submit" value="submit"' +
        ' tabindex="2" accesskey="s" />' +
        '<input type="hidden" name="mode" value="search" />' +
        '</p></form>\n')
    if key:
        count = config.search_count
        print "<ul>"
        buf = geturl("http://find.2ch.net/?BBS=ALL&TYPE=TITLE&" +
                     "STR=%s&COUNT=%d&OFFSET=%d" % (key, count, offset))
        n = 0
        getresult = re.compile(
            r"^<dt><a href=[\"']?([^<>]*).*<b>([^>]+)</b></a> \((\d+)\)")
        parseurl = re.compile(
            r"http://([^/]+)/test/read\.cgi/([^/]+)/(\d+)")
        for line in buf:
            found = getresult.search(line)
            if not found:
                continue
            url, title, size = found.groups()
            m = parseurl.search(url)
            if not m:
                continue
            host, board, key = m.groups()
            print '<li><a href="%s?http://%s/%s/dat/%s.dat">%s (%s)</a></li>' \
                  % (script, host, board, key, title, size)
            n += 1
        print "</ul><p>"
        if offset:
            print '<a href="%s?mode=search&amp;key=%s&amp;off=%d">Prev</a>' \
                  % (script, str_key, offset-count)
        else:
            print "...."
        print " | "
        if n >= count:
            print '<a href="%s?mode=search&amp;key=%s&amp;off=%d">Next</a>' \
                  % (script, str_key, offset+count)
        else:
            print "...."
        print "</p>"
    print "</body></html>"
